/*  Copyright 2011 diwaly
 *  This file is part of Volcano Plugin.
 *
 *  Volcano Plugin is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Volcano Plugin is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Volcano Plugin.  If not, see <http://www.gnu.org/licenses/>.
 */
package fr.diwaly.volcano;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Scanner;
import java.util.Map.Entry;
import java.util.TreeMap;
import java.util.logging.Logger;

import org.bukkit.ChatColor;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Server;
import org.bukkit.World;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.java.JavaPlugin;

public class Plugin extends JavaPlugin implements Runnable{
	static private final ArrayList<Player> debugees = new ArrayList<Player>();
	static final private Logger log = Logger.getLogger("Minecraft");
	static private String debug = "";
	public static File dataFolder;
    static public ArrayList<Volcano> listVolcano;
    static private TreeMap<String, Double> allowsBlocks;
    static private ArrayList<String> allowsWorlds;
    static public Server server;
    static private boolean forOps = true;
    static public Plugin him;
    
    public Plugin(){
    }
    
	@Override
	public void onDisable() {	
	}

	@Override
	public void onEnable() {
		him = this;
		server = this.getServer();
		listVolcano = new ArrayList<Volcano>();
		
		allowsBlocks = new TreeMap<String, Double>();
		allowsWorlds = new ArrayList<String>();
		dataFolder = this.getDataFolder();
		dataFolder.mkdir();
		
		server.getScheduler().scheduleSyncRepeatingTask(Plugin.this, this,50,20); // tick second
		
		new VolcanoLavaListener();
		new VolcanoPlayerListener();
		new VolcanoActive();
		
		logDescription();
	}

	@Override
	public boolean onCommand(CommandSender sender, Command command,	String label, String[] args) {		
		if(!(sender instanceof Player))
    		return false;
    	Player player = (Player)sender;
    	String display = "";
    	for (int i = 0; i < args.length; i++) {
    		display+=" "+args[i];
		}
    	log(player.getName()+":"+display);
    	
    	if(label.equalsIgnoreCase("volcano") && args.length > 0){
    		boolean permit = true;
    		if(forOps)
    			permit = player.isOp();
    		if(permit){
	    		try {
	    			String cmd = (args == null)?"":args[0];   
					int length = args.length;
					if(cmd.equalsIgnoreCase("help")){
						return help(player, length, args);
					}else if(cmd.equalsIgnoreCase("help2")){
						return help2(player, length, args);
					}else if(cmd.equalsIgnoreCase("create")){
						return create(player, length, args);
					}else if(cmd.equalsIgnoreCase("layer")){
						return layer(player, length, args);
					}else if(cmd.equalsIgnoreCase("layerHelp")){
						return layerHelp(player, length, args);
					}else if(cmd.equalsIgnoreCase("layerInfo")){
						return layerInfo(player, length, args);
					}else if(cmd.equalsIgnoreCase("start")){
						return start(player, length, args);
					}else if(cmd.equalsIgnoreCase("stop")){
						return stop(player, length, args);
					}else if(cmd.equalsIgnoreCase("list")){
						return list(player, length, args);
					}else if(cmd.equalsIgnoreCase("del")){
						return del(player, length, args);
					}else if(cmd.equalsIgnoreCase("load")){
						return load(player, length, args);
					}else if(cmd.equalsIgnoreCase("crater")){
						return crater(player, length, args);	
					}else if(cmd.equalsIgnoreCase("timer")){
						return timer(player, length, args);
					}else if(cmd.equalsIgnoreCase("flowed")){
						return flowed(player, length, args);
					}else if(cmd.equalsIgnoreCase("erupt")){
						return erupt(player, length, args);
					}else if(cmd.equalsIgnoreCase("timerExplo")){
						return timerExplo(player, length, args);
					}else if(cmd.equalsIgnoreCase("delayExplo")){
						return delayExplo(player, length, args);
					}else if(cmd.equalsIgnoreCase("nbExplo")){
						return nbExplo(player, length, args);
					}else if(cmd.equalsIgnoreCase("info")){
						return info(player, length, args);
					}else if(cmd.equalsIgnoreCase("move")){
						return move(player, length, args);
					}else if(cmd.equalsIgnoreCase("near")){
						return near(player, length, args);
					}else if(cmd.equalsIgnoreCase("allow")){
						return allow(player, length, args);
					}else if(cmd.equalsIgnoreCase("world")){
						return world(player, length, args);
					}else if(cmd.equalsIgnoreCase("ops")){
						return ops(player, length, args);
					}else if(cmd.equalsIgnoreCase("modlayer")){
						return modlayer(player, length, args);
					}else if(cmd.equalsIgnoreCase("dellayer")){
						return dellayer(player, length, args);	
					}
				} catch (Exception e) {
					log("Error on command: "+e.getMessage());
					for (StackTraceElement stack : e.getStackTrace()) {
						log("Error on command: "+stack.toString());
					}			
				} 
				player.sendMessage(ChatColor.RED+"Bad command.");
		    	return false;
	    	}else{
	    		player.sendMessage(ChatColor.GRAY+"Volcano commands are for ops.");
	    		return true;
	    	}
		}
    	return false;
	}
	
	static public Volcano giveVolcano(String name){
		for (Volcano volcano : listVolcano) {
			if(volcano.name.equals(name)){
				return volcano;
			}
		}
		return null;
	}
	
	static final public long getTime(){
		return System.currentTimeMillis();
	}
	
	@Override
	public void run() {
		for (Volcano volcano : listVolcano) {
			volcano.update();
			volcano.check();
		}
	}
	
	static public void load(){
		blocksPermit();
		worldPermit();
		
		// Add volcano files
		File dir = Volcano.getDir();
		for (File file : dir.listFiles()) {
			if(file.isFile())
				listVolcano.add(new Volcano(file));
		}		
		// Permition ops
		File prop = getPropertieFile();
		if(!prop.exists())
			savePropperties();
		loadPropperties();
	}
	
	static private void worldPermit(){	
		File fileWorld = getWorldFile();		
		try {
			if(!fileWorld.exists()){
				for (World w : server.getWorlds()) {
					allowsWorlds.add(w.getName());
				}
				saveWorld();
			}else{
				Scanner scan = new Scanner(fileWorld);
				while (scan.hasNext()) {
					allowsWorlds.add(scan.next());
				}
				scan.close();
			}
		} catch (Exception e) {
			Plugin.log("Error load worlds permission : "+e.getMessage());
		}				
	}
	
	static private void blocksPermit(){
		dataFolder.mkdir();
		File listBlocks = new File(dataFolder, "listBlocks.txt");
		File allow = new File(dataFolder, "allowBlock.txt");
		String nameB = "";
		try {
			if(!listBlocks.exists()){			
				BufferedWriter wList = new BufferedWriter(new FileWriter(listBlocks));
				Material[] listMat = Material.values();
				ArrayList<String> listMatName = new ArrayList<String>();
				for (int i=0 ; i < listMat.length ; i++) {
					if(listMat[i].isBlock())
						listMatName.add(listMat[i].name());
				}
				Collections.sort(listMatName);
				for (String name : listMatName) {
						wList.append(name);
						wList.newLine();
				}
				wList.close();
			}
			if(!allow.exists()){
				BufferedWriter wAllow = new BufferedWriter(new FileWriter(allow));
				wAllow.append(Material.AIR.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.PUMPKIN.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.NETHERRACK.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.DIRT.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.SOUL_SAND.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.GLOWSTONE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.JACK_O_LANTERN.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.GOLD_BLOCK.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.IRON_BLOCK.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.BRICK.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.MOSSY_COBBLESTONE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.OBSIDIAN.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.MOB_SPAWNER.name()+" 0.5");
				wAllow.newLine();
				wAllow.append(Material.DIAMOND_ORE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.DIAMOND_BLOCK.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.REDSTONE_ORE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.CLAY.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.COAL_ORE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.GLASS.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.LAPIS_ORE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.LAPIS_BLOCK.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.SANDSTONE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.WEB.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.GRAVEL.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.GOLD_ORE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.IRON_ORE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.STONE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.COBBLESTONE.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.SAND.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.TNT.name()+" 0.5");
				wAllow.newLine();
				wAllow.append(Material.GRASS.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.HUGE_MUSHROOM_1.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.HUGE_MUSHROOM_2.name()+" 100");
				wAllow.newLine();
				wAllow.append(Material.MELON_BLOCK.name()+" 10");
				wAllow.newLine();
				wAllow.append(Material.MONSTER_EGGS.name()+" 100");
				wAllow.newLine();
				wAllow.close();
				
			}
			Scanner scan = new Scanner(allow);
			while (scan.hasNext()) {
				nameB = scan.next();
				allowsBlocks.put(nameB, Double.valueOf(scan.next()));
			}
			scan.close();
			
		} catch (Exception e) {
			Plugin.log("Error blocks permission : "+e.getMessage()+" sur "+nameB);
			e.printStackTrace();
		}
	}
	
	static final public boolean isNearVolcano(int x, int z){
		for (Volcano v : listVolcano) {
			if(v.insideChunk(x, z))
				return true;
		}
		return false;
	}
	
	static final public ArrayList<String> nearVolcano(int x, int z){
		ArrayList<String> nears = new ArrayList<String>();
		for (Volcano volcano : listVolcano) {
			if(volcano.insideChunk(x, z)){
				nears.add(volcano.name);
			}
		}
		return nears;
	}
	
	static final private boolean setWorld(String world){			
		 if(allowsWorlds.contains(world) || (server.getWorld(world) == null)){
			 allowsWorlds.remove(world);
			 saveWorld();
			return false;
		}else{
			allowsWorlds.add(world);
			saveWorld();
			return true;
		}
	}
	static final public void saveWorld(){
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(getWorldFile()));
			for (String w : allowsWorlds) {
				writer.append(w);
				writer.newLine();			
			}
			writer.close();
		} catch (Exception e) {
			Plugin.log("Error save worlds permission : "+e.getMessage());
		}
	}
	
	static final public void savePropperties(){
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(getPropertieFile()));
			writer.append("onlyOps: "+forOps);
			writer.close();
			
		} catch (Exception e) {
			Plugin.log("Error save properties : "+e.getMessage());
		}
	}
	static final public void loadPropperties(){
		try {
			Scanner scan = new Scanner(getPropertieFile());
			scan.next();
			forOps = scan.nextBoolean();
			scan.close();
		} catch (Exception e) {
			Plugin.log("Error load properties : "+e.getMessage());
		}
	}
	
	static final public File getWorldFile(){
		return new File(dataFolder, "allowWorld.txt");
	}
	static final public File getPropertieFile(){
		return new File(dataFolder, "volcano.properties");
	}
	
	static final public void setDebugging(Player player){
		debugees.add(player);
	}
	
	static final public void debug(String msg){
		debug(msg, false);
	}
	static final public void debug(String msg, boolean sameLine){
		debug = debug+msg+" ";
		if(!sameLine){
			for (Player player : debugees) {
				player.sendMessage(ChatColor.AQUA+debug);
			}
			debug = "";
		}
	}
	static public void logDescription(){
		PluginDescriptionFile pdfFile = him.getDescription();
        log( pdfFile.getName() +" "+pdfFile.getVersion() + " is enabled!" );
	}
	static public void log(String msg){
		log.info("[Volcano] "+msg);
	}
	
	final public String strDontExist = " doesn't exist.";
	final public String strLayerModify = " layer was modified.";
	final public String strLayerAdd = " layer was added.";
	final public String str100Blocks = "100% of the blocks must be filled.";
	
	public boolean help(Player player, int length, String[] args){
		if(length == 1){
			player.sendMessage(ChatColor.GRAY+"*** HELP ***");
			player.sendMessage(ChatColor.GRAY+"/volcano create <name> <height> <rate nameBloc> ... - Create volcano.");
			player.sendMessage(ChatColor.GRAY+"/volcano start <name> - Start a volcano.");
			player.sendMessage(ChatColor.GRAY+"/volcano stop <name> - Stop a volcano.");	    					
			player.sendMessage(ChatColor.GRAY+"/volcano crater <name> [<radius>] - Crater radius (between 1 and 100).");
			player.sendMessage(ChatColor.GRAY+"/volcano timer <name> [<second>] - Time between flowed.");
			player.sendMessage(ChatColor.GRAY+"/volcano flowed <name> [<second>] - Time of flowed before transformation.");
			player.sendMessage(ChatColor.GRAY+"/volcano erupt <name> explo|effu - Mode of eruption  : Explosive or  Effusive (without explosion).");
			player.sendMessage(ChatColor.GRAY+"/volcano timerExplo <name> [<second>] - Fixe timer before explosion once at the top.");
			player.sendMessage(ChatColor.GRAY+"/volcano delayExplo <name> [<second>] - Random delay after the fixe timer.");
			player.sendMessage(ChatColor.GRAY+"/volcano nbExplo <name> [<number>] - Number of explosion.");
			player.sendMessage(ChatColor.GRAY+"/volcano info <name> - Information about a volcano.");
			player.sendMessage(ChatColor.GRAY+"/volcano move <name> - Move a volcano on new position (not the blocks).");
			player.sendMessage(ChatColor.GRAY+"/volcano help2 - Other commands.");
			return true;
		}
		return false;
	}
	public boolean help2(Player player, int length, String[] args){
		if(length == 1){
			player.sendMessage(ChatColor.GRAY+"*** HELP2 ***");
			player.sendMessage(ChatColor.GRAY+"/volcano layerHelp - Help for add the layers.");
			player.sendMessage(ChatColor.GRAY+"/volcano del <name> - Delete a volcano file (not the blocks in game).");
			player.sendMessage(ChatColor.GRAY+"/volcano load <name> - Load a volcano file.");			
			player.sendMessage(ChatColor.GRAY+"/volcano list - List of volcanoes.");
			player.sendMessage(ChatColor.GRAY+"/volcano near - Display the near volcanoes.");
			player.sendMessage(ChatColor.GRAY+"/volcano allow [<from>] - List of allow blocks.");
			if(player.isOp()){
				player.sendMessage(ChatColor.GRAY+"/volcano world <nameWorld> - Allow/Forbidden a world for the volcano plugin. (op)");
				player.sendMessage(ChatColor.GRAY+"/volcano ops - Permit the volcano plugin only for ops or allow for everybody. (op)");
			}
			return true;
		}
		return false;
	}
	
	
	public boolean create(Player player, int length, String[] args){
		if(length > 3){
			if(allowsWorlds.contains(player.getWorld().getName())){
				String name = args[1];
				Volcano volcano = giveVolcano(name); 
				if(volcano == null){
	    			int x = player.getLocation().getBlockX();
	    			int y = player.getLocation().getBlockY();
	    			int z = player.getLocation().getBlockZ();
	    			int maxY = Integer.valueOf(args[2]) + y;
	    			if(!isNearVolcano(x, z)){
	    				volcano = new Volcano(player.getWorld(),name, maxY, x, y, z);
	        			volcano.setExplosive();
	        			listVolcano.add(volcano);
	        			int begin = 3;
	        			int numLayer = -1;
	        			int radius = 0;
	        			if(modLayer(player, length, args, begin, numLayer, radius)){
	        				player.sendMessage(ChatColor.GRAY+"Volcano "+name+" created.");
	        			}else{
	        				listVolcano.remove(volcano);
	        			}
	        			return true;
	    			}else{
	    				player.sendMessage(ChatColor.RED+"You are near a volcano.");
	    				return true;
	    			}
				}else{
					player.sendMessage(ChatColor.RED+"Volcano "+name+" exists.");
					return true;
				}
			}else{
				player.sendMessage(ChatColor.RED+"Create a volcano on "+player.getWorld().getName()+" isn't allowed.");
				return true;
			}
		}
		return false;
	}
	public boolean layer(Player player, int length, String[] args){
		if(length > 3){
			String name = args[1];
			int begin = 3;
			int numLayer=-1;
			int radius = Integer.valueOf(args[2]);
			if(modLayer(player, length, args, begin, numLayer, radius)){
				player.sendMessage(ChatColor.GRAY+name+strLayerModify);
			}
			return true;
		}
		return false;
	}
	public boolean start(Player player, int length, String[] args){
		if(length == 2){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				if(volcano.layerIsinit()){
					volcano.initFirstStart();
					volcano.start();
					volcano.save();
					player.sendMessage(ChatColor.GRAY+name+" starts.");
				}else{
					player.sendMessage(ChatColor.GRAY+name+" haven't of layer.");
				}
				return true;
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean stop(Player player, int length, String[] args){
		if(length == 2){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){	
				volcano.stop();
				volcano.save();
				player.sendMessage(ChatColor.GRAY+name+" is stopped.");
				return true;       				
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean list(Player player, int length, String[] args){
		if(length == 1){
			player.sendMessage(ChatColor.GRAY+"List of volcanoes :");
			for (Volcano volcano : listVolcano) {
				player.sendMessage(ChatColor.GRAY+"- "+volcano.name+" "+String.valueOf(volcano.isStart()));
			}
			return true;
		}
		return false;
	}
	public boolean del(Player player, int length, String[] args){
		if(length == 2){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				volcano.enable = false;
				new VolcanoDelete(volcano);
				player.sendMessage(ChatColor.GRAY+name+" deleting is pending.");
				return true;
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean load(Player player, int length, String[] args){
		if(length == 2){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano == null){
				volcano = new Volcano(Volcano.getFile(name));
				listVolcano.add(volcano);
			}else{
				volcano.load();
				volcano.initHeight();
			}
			player.sendMessage(ChatColor.GRAY+name+" is loaded.");
			return true;
		}
		return false;
	}
	public boolean timer(Player player, int length, String[] args){
		if(length > 1){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				if(length == 2){
					player.sendMessage(ChatColor.GRAY+name+" timer is "+volcano.getTimerBetweenFlowed());
					return true;
				}else if(length == 3){
					String second = args[2];
					volcano.initTimerBetweenFlowed(Long.valueOf(second));
					player.sendMessage(ChatColor.GRAY+name+" timer : "+second);
					volcano.save();
					return true;
				}
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean flowed(Player player, int length, String[] args){
		if(length > 1){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				if(length == 2){
					player.sendMessage(ChatColor.GRAY+name+" flowed is "+volcano.getTimerFlowed());
					return true;
				}else if(length == 3){
					String second = args[2];
					volcano.initTimerFlowed(Long.valueOf(second));
					player.sendMessage(ChatColor.GRAY+name+" flowed : "+second);
					volcano.save();
					return true;
				}
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean erupt(Player player, int length, String[] args){
		if(length == 3){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				String mode = args[2];
				if(mode.equalsIgnoreCase("explo")){
					volcano.setExplosive();
					volcano.save();
					player.sendMessage(ChatColor.GRAY+name+" has explosive eruption.");
				}else if(mode.equalsIgnoreCase("effu")){
					volcano.setEffusive();
					volcano.save();
					player.sendMessage(ChatColor.GRAY+name+" has effusive eruption.");
				}else{
					player.sendMessage(ChatColor.GRAY+"/volcano erupt <name> explo or effu");
				}
				return true;
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean timerExplo(Player player, int length, String[] args){
		if(length > 1){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				if(length == 2){
					player.sendMessage(ChatColor.GRAY+name+" timer before explosion is "+volcano.getTimerExplo());
					return true;
				}else if(length == 3){
					String timer = args[2];
					volcano.setTimerExplo(Integer.valueOf(timer));
					player.sendMessage(ChatColor.GRAY+name+" timer explosion : "+timer);
					volcano.save();
					return true;
				}
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean delayExplo(Player player, int length, String[] args){
		if(length > 1){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				if(length == 2){
					player.sendMessage(ChatColor.GRAY+name+" delay before explosion is "+volcano.getDelayExplo());
					return true;
				}else if(length == 3){
					String delay = args[2];
					volcano.setDelayExplo(Integer.valueOf(delay));
					player.sendMessage(ChatColor.GRAY+name+" delay explosion : "+delay);
					volcano.save();
					return true;
				}
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean nbExplo(Player player, int length, String[] args){
		if(length > 1){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				if(length == 2){
					player.sendMessage(ChatColor.GRAY+name+" number of explosion is "+volcano.nbExplo);
					return true;
				}else if(length == 3){
					String nb = args[2];
					volcano.nbExplo = Integer.valueOf(nb);
					player.sendMessage(ChatColor.GRAY+name+" number explosion : "+nb);
					volcano.save();
					return true;
				}
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean info(Player player, int length, String[] args){
		if(length == 2){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				player.sendMessage(ChatColor.GRAY+"*** Information about "+volcano.name+" ***");
				player.sendMessage(ChatColor.GRAY+"* enable : "+String.valueOf(volcano.isStart()));
				player.sendMessage(ChatColor.GRAY+"* world : "+volcano.world.getName());
				player.sendMessage(ChatColor.GRAY+"* crater : "+volcano.crater);
				player.sendMessage(ChatColor.GRAY+"* eruption : "+((volcano.explosive)?"Explosive":"Effusive"));
				player.sendMessage(ChatColor.GRAY+"* timerExplo : "+volcano.getTimerExplo());
				player.sendMessage(ChatColor.GRAY+"* delayExplo : "+volcano.getDelayExplo());
				player.sendMessage(ChatColor.GRAY+"* nbExplo : "+volcano.nbExplo);
				player.sendMessage(ChatColor.GRAY+"* refPointX : "+volcano.refPointX);
				player.sendMessage(ChatColor.GRAY+"* refPointY : "+volcano.refPointY);
				player.sendMessage(ChatColor.GRAY+"* refPointZ : "+volcano.refPointZ);
				player.sendMessage(ChatColor.GRAY+"* maxY : "+volcano.maxY);
				player.sendMessage(ChatColor.GRAY+"* timerBetweenFlowed : "+volcano.getTimerBetweenFlowed());
				player.sendMessage(ChatColor.GRAY+"* timerFlowed : "+volcano.getTimerFlowed());
				player.sendMessage(ChatColor.GRAY+"*** END ***");
				return true;
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean move(Player player, int length, String[] args){
		if(length == 2){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				int x = player.getLocation().getBlockX();
    			int y = player.getLocation().getBlockY();
    			int z = player.getLocation().getBlockZ();
    			int maxY = y-volcano.refPointY+volcano.maxY;
    			if(!isNearVolcano(x, z)){
    				volcano.world = player.getWorld();
    				volcano.maxY = maxY;
    				volcano.refPointX = x;
    				volcano.refPointY = y;
    				volcano.refPointZ = z;
    				volcano.initCaldeira();
    				volcano.save();
        			player.sendMessage(ChatColor.GRAY+"Volcano "+name+" was moved here.");
        			return true;
    			}else{
    				player.sendMessage(ChatColor.RED+"You are near a volcano.");
    				return true;
    			}
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean near(Player player, int length, String[] args){
		if(length == 1){
			player.sendMessage(ChatColor.GRAY+"*** NEAR ***");
			Location loc = player.getLocation();
			for (String name : nearVolcano(loc.getBlockX(), loc.getBlockZ())) {
				player.sendMessage(ChatColor.GRAY+"* "+name);
			}
			return true;
		}
		return false;
	}
	public boolean allow(Player player, int length, String[] args){
		int page = 0;
		int i = 1;
		int end = allowsBlocks.size();
		if(length == 2){
			page = Integer.valueOf(args[1]);  
			end = page +16;
		}
		player.sendMessage(ChatColor.GRAY+"*** ALLOW ***");
		for (Entry<String, Double> allow : allowsBlocks.entrySet()) {
			if (i > page && i < end){
				if(allow.getValue() == 100){
					player.sendMessage(ChatColor.GRAY+"* block: "+allow.getKey());
				}else{
					player.sendMessage(ChatColor.GRAY+"* block: "+allow.getKey()+" rate: "+allow.getValue());
				}
			}
			i++;
		}		
		return true;
	}
	public boolean world(Player player, int length, String[] args){
		if(length == 2 && player.isOp()){
			String name = args[1];
			if(setWorld(name)){
				player.sendMessage(ChatColor.GREEN+"World "+name+" is allowed for the volcano plugin.");	
			}else{
				player.sendMessage(ChatColor.RED+"World "+name+" isn't allowed for the volcano plugin.");
			}
			return true;
		}
		return false;
	}
	public boolean ops(Player player, int length, String[] args){
		if(length == 1){
			if(forOps){
				forOps = false;
				player.sendMessage(ChatColor.GREEN+"Free for all !");
			}else{
				forOps = true;
				player.sendMessage(ChatColor.RED+"Ops only.");
			}
			savePropperties();
			return true;
		}
		return false;
	}
	public boolean layerHelp(Player player, int length, String[] args){
		if(length == 1 ){
			player.sendMessage(ChatColor.GRAY+"*** LAYER HELP ***");
			player.sendMessage(ChatColor.GRAY+"/volcano layer <name> <radius> <rate nameBloc> ... - Add layer.");
			player.sendMessage(ChatColor.GRAY+"/volcano layerInfo <name> - Info layer.");
			player.sendMessage(ChatColor.GRAY+"/volcano modlayer <name> <num> <radius> <rate nameBloc> ... - Modification of layer.");
			player.sendMessage(ChatColor.GRAY+"/volcano dellayer <name> <num> - Delete layer.");
			return true;
		} 
		return false;
	}
	public boolean layerInfo(Player player, int length, String[] args){
		if(length == 2){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				player.sendMessage(ChatColor.GRAY+"*** Information about "+volcano.name+" layer ***");	
				for(int num = 0 ; num < volcano.popLayer.length ; num++){
					player.sendMessage(ChatColor.GRAY+"* Layer"+num+" radius: "+volcano.layerRadius[num]);
					PopBlock popy = volcano.popLayer[num];
					int size = popy.size();
					for (int i=0 ; i < size ; i++) {
						player.sendMessage(ChatColor.GRAY+"* pop : "+popy.getRateH(i)+" % "+popy.getMat(i));
					}
				}
				player.sendMessage(ChatColor.GRAY+"*** END ***");
				return true;
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean modlayer(Player player, int length, String[] args){
		if(length > 4){
			String name = args[1];
			int begin = 4;
			int numLayer = Integer.valueOf(args[2]);
			int radius = Integer.valueOf(args[3]);
			if(modLayer(player, length, args, begin, numLayer, radius)){
				player.sendMessage(ChatColor.GRAY+name+strLayerModify);
			}
			return true;
		}
		return false;
	}
	public boolean dellayer(Player player, int length, String[] args){
		if(length == 3){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				int num = Integer.valueOf(args[2]);
				if(volcano.popLayer.length > 1){
					if(volcano.delLayer(num)){
						volcano.save();
						player.sendMessage(ChatColor.GRAY+"Delete layer "+num+" on "+name+".");
					}else{
						player.sendMessage(ChatColor.GRAY+"Bad layer.");
					}
				}else {
					player.sendMessage(ChatColor.GRAY+"It remain one layer. You can't delete it.");
				}	
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
			}
			return true;
		}
		return false;
	}	
	public boolean crater(Player player, int length, String[] args){
		if(length > 1){
			String name = args[1];
			Volcano volcano = giveVolcano(name);
			if(volcano != null){
				if(length == 2){
					player.sendMessage(ChatColor.GRAY+name+" crater radius is "+volcano.crater);
					return true;
				}else if(length == 3){
					int radius = Integer.valueOf(args[2]);
					if(radius > 0 && radius < 101){
						volcano.crater = radius;
						volcano.initCaldeira();
						player.sendMessage(ChatColor.GRAY+name+" crater : "+radius);
						volcano.save();
						return true;
					}else{
						player.sendMessage(ChatColor.GRAY+"The crater radius is between 1 and 100.");
						return true;
					}
				}
			}else{
				player.sendMessage(ChatColor.GRAY+"Volcano "+name+strDontExist);
				return true;
			}
		}
		return false;
	}
	public boolean debug(Player player, int length, String[] args){
		if(length == 1){
			if(debugees.contains(player)){
				debugees.remove(player);
				player.sendMessage(ChatColor.RED+"Debug disable.");
			}else{
				debugees.add(player);
				player.sendMessage(ChatColor.GREEN+"Debug enable.");
			}
			return true;
		}
		return false;
	}
	
	public boolean modLayer(Player player, int length, String[] args, int begin, int numLayer, int radius){
		String name = args[1];
		Volcano volcano = giveVolcano(name);
		if(volcano != null){	
			PopBlock popy = new PopBlock();
			double sum = 0;		        			
			for (int i = begin ; i < length ; i++){
				double rate = Double.valueOf(args[i]);
				String nameMat = args[++i].toUpperCase();
				sum += rate;
				Material mat = Material.getMaterial(nameMat);
				if(mat != null){
					if(mat.isBlock()){
						if(allowsBlocks.containsKey(nameMat) && rate <= allowsBlocks.get(nameMat)){
							popy.put(rate, mat);
						}else{
							player.sendMessage(ChatColor.RED+"Material "+nameMat+" is forbidden.");
							return false;
						}
					}else{
						player.sendMessage(ChatColor.RED+nameMat+" isn't a block.");
						return false;
					}
				}else{
					player.sendMessage(ChatColor.RED+"Material "+nameMat+" doesn't exist.");
					return false;
				}
			}
			if(sum == 100){
				if(numLayer < 0){
					volcano.addLayer(radius, popy.machine());
				}else{
					volcano.addLayer(numLayer, radius, popy.machine());
				}
    			return true;
			}else{
				player.sendMessage(ChatColor.RED+str100Blocks);
				return false;
			}	
		}else{
			player.sendMessage(ChatColor.RED+"Volcano "+name+" don't exist.");
			return false;
		}	
	}	
}
